"""Entry point for ``python -m icakad``."""

from __future__ import annotations

import sys
import webbrowser
from pathlib import Path
from typing import Optional

from .cli import main as cli_main

DOCS_URL = "https://linkove.icu/docs/icakad/"


def _find_local_docs() -> Optional[Path]:
    current = Path(__file__).resolve()
    for parent in current.parents:
        candidate = parent / "docs" / "index.html"
        if candidate.exists():
            return candidate
    return None


def run() -> int:
    if len(sys.argv) == 1:
        docs_path = _find_local_docs()
        if docs_path:
            print("Open the local documentation:", docs_path)
            try:  # pragma: no cover - optional UX improvement
                webbrowser.open(docs_path.as_uri())
            except Exception:
                pass
        else:
            print("Documentation: ", DOCS_URL)
        print("Use 'python -m icakad --help' for command examples.")
        return 0

    return cli_main(sys.argv[1:])


if __name__ == "__main__":  # pragma: no cover
    sys.exit(run())



import json
import os
import tempfile
import unittest
from pathlib import Path
from unittest.mock import MagicMock, patch

import requests

from icakad.config import load_settings
from icakad.paste import PasteClient
from icakad.shorturl import ShortURLClient
from icakad import (
    AI,
    create_paste,
    delete_short_link,
    fetch_paste,
    list_short_links,
    update_short_link,
)


def make_response(*, json_data=None, text="", status=200):
    response = MagicMock()
    response.status_code = status

    if status >= 400:
        response.raise_for_status.side_effect = requests.HTTPError("error")
    else:
        response.raise_for_status.return_value = None

    if json_data is not None:
        response.json.return_value = json_data
    else:
        response.json.side_effect = ValueError("no json")

    response.text = text
    return response


class SettingsTests(unittest.TestCase):
    def test_load_settings_from_file_and_overrides(self):
        with tempfile.TemporaryDirectory() as tmp:
            config_path = Path(tmp) / "icakad.json"
            config_path.write_text(json.dumps({"token": "file-token"}), encoding="utf-8")

            os.environ["ICAKAD_SHORTURL_BASE"] = "https://env-short.test"
            try:
                settings = load_settings(
                    config_path=config_path,
                    paste_base="https://override-paste.test",
                )
            finally:
                os.environ.pop("ICAKAD_SHORTURL_BASE", None)

        self.assertEqual(settings.token, "file-token")
        self.assertEqual(settings.shorturl_base, "https://env-short.test")
        self.assertEqual(settings.paste_base, "https://override-paste.test")


class ShortURLClientTests(unittest.TestCase):
    def test_client_operations_use_expected_routes(self):
        session = MagicMock()
        base = "https://linkove.icu"
        session.post.side_effect = [
            make_response(json_data={"ok": True}),
            make_response(json_data={"ok": True}),
        ]
        session.delete.return_value = make_response(json_data={"ok": True})
        session.get.return_value = make_response(
            json_data={"items": [{"slug": "demo", "url": "https://example.com"}]}
        )

        client = ShortURLClient(base_url=base, token="token", session=session)
        client.add_link("demo", "https://example.com")
        session.post.assert_called_with(
            f"{base}/api",
            json={"slug": "demo", "url": "https://example.com"},
            headers={"Content-Type": "application/json", "Authorization": "Bearer token"},
            timeout=10,
        )

        client.edit_link("demo", "https://example.com")
        session.post.assert_called_with(
            f"{base}/api/demo",
            json={"slug": "demo", "url": "https://example.com"},
            headers={"Content-Type": "application/json", "Authorization": "Bearer token"},
            timeout=10,
        )

        client.delete_link("demo")
        session.delete.assert_called_with(
            f"{base}/api/demo",
            headers={"Content-Type": "application/json", "Authorization": "Bearer token"},
            timeout=10,
        )

        links = client.list_links()
        self.assertEqual(links, {"demo": "https://example.com"})


class PasteClientTests(unittest.TestCase):
    def test_fetch_paste_merges_metadata(self):
        session = MagicMock()
        base = "https://linkove.icu"
        session.post.return_value = make_response(json_data={"ok": True, "id": "abc", "url": f"{base}/abc"})
        session.get.side_effect = [
            make_response(text="hello"),
            make_response(json_data={"ok": True, "pastes": [{"id": "abc", "text": "hello"}]}),
        ]

        client = PasteClient(base_url=base, session=session)
        created = client.create_paste("hello")
        self.assertTrue(created["ok"])
        fetched = client.fetch_paste("abc")
        self.assertEqual(fetched["text"], "hello")
        self.assertEqual(fetched["id"], "abc")
        self.assertEqual(fetched["url"], f"{base}/abc")


class HighLevelHelpersTests(unittest.TestCase):
    def test_list_short_links_can_write_file(self):
        fake_client = MagicMock()
        fake_client.list_links.return_value = {"demo": "https://example.com"}
        with tempfile.TemporaryDirectory() as tmp, patch("icakad._client_from_settings", return_value=fake_client):
            output = Path(tmp) / "links.json"
            data = list_short_links(save_to=output)
            self.assertEqual(data, {"demo": "https://example.com"})
            stored = json.loads(output.read_text(encoding="utf-8"))
            self.assertEqual(stored, data)

    def test_fetch_paste_raw_output(self):
        fake_client = MagicMock()
        fake_client.fetch_paste.return_value = "hello world"
        with tempfile.TemporaryDirectory() as tmp, patch("icakad._paste_client_from_settings", return_value=fake_client):
            output = Path(tmp) / "paste.txt"
            result = fetch_paste("abc", raw=True, save_to=output)
            self.assertEqual(result, "hello world")
            self.assertEqual(output.read_text(encoding="utf-8"), "hello world")

    def test_create_paste_uses_text_file(self):
        fake_client = MagicMock()
        fake_client.create_paste.return_value = {"ok": True}
        with tempfile.TemporaryDirectory() as tmp, patch("icakad._paste_client_from_settings", return_value=fake_client):
            text_file = Path(tmp) / "note.txt"
            text_file.write_text("Hello", encoding="utf-8")
            result = create_paste(text_file=text_file)
            self.assertTrue(result["ok"])
            fake_client.create_paste.assert_called_with("Hello", paste_id=None, ttl=None, as_plaintext=False)

    def test_update_and_delete_short_link(self):
        fake_client = MagicMock()
        fake_client.edit_link.return_value = {"ok": True}
        fake_client.delete_link.return_value = {"ok": True}

        with patch("icakad._client_from_settings", return_value=fake_client):
            update = update_short_link("demo", "https://example.com")
            delete = delete_short_link("demo")
        self.assertTrue(update["ok"])
        self.assertTrue(delete["ok"])


class AITests(unittest.TestCase):
    def test_ask_posts_prompt_to_worker(self):
        with patch("icakad.ai.requests.post") as mocked_post:
            mocked_post.return_value = make_response(
                json_data={"response": "Здрасти"}
            )
            reply = AI.ask("Hello from Pythonista")

        self.assertEqual(reply, "Здрасти")
        mocked_post.assert_called_with(
            "https://llama.icakad.workers.dev/",
            json={"messages": [{"role": "user", "content": "Hello from Pythonista"}]},
            headers={"Content-Type": "application/json"},
            timeout=20.0,
        )


if __name__ == "__main__":
    unittest.main()



"""Configuration helpers for icakad clients."""

from __future__ import annotations

import json
import os
from dataclasses import dataclass, replace
from pathlib import Path
from typing import Any, Dict, Optional

DEFAULT_SHORTURL_BASE = "https://linkove.icu"
DEFAULT_PASTE_BASE = "https://linkove.icu"

DEFAULT_CONFIG_LOCATIONS = (
    Path.home() / ".config" / "icakad" / "config.json",
    Path.cwd() / "icakad.config.json",
)


@dataclass(frozen=True)
class Settings:
    """Simple container describing API configuration."""

    shorturl_base: str = DEFAULT_SHORTURL_BASE
    paste_base: str = DEFAULT_PASTE_BASE
    token: Optional[str] = None

    def with_overrides(self, **overrides: Any) -> "Settings":
        """Return a copy with any non-None overrides applied."""
        current: Dict[str, Any] = {
            "shorturl_base": self.shorturl_base,
            "paste_base": self.paste_base,
            "token": self.token,
        }
        for key, value in overrides.items():
            if value is not None and key in current:
                current[key] = value
        return Settings(**current)


def _load_json(path: Path) -> Dict[str, Any]:
    with path.open("r", encoding="utf-8") as fh:
        return json.load(fh)


def load_settings(
    config_path: Optional[str | Path] = None,
    *,
    token: Optional[str] = None,
    shorturl_base: Optional[str] = None,
    paste_base: Optional[str] = None,
) -> Settings:
    """Load settings from config files, environment variables, and overrides."""
    settings = Settings()

    # Config file from explicit path or environment hint.
    candidate_paths = []
    if config_path:
        candidate_paths.append(Path(config_path).expanduser())
    env_path = os.environ.get("ICAKAD_CONFIG")
    if env_path:
        candidate_paths.append(Path(env_path).expanduser())
    candidate_paths.extend(DEFAULT_CONFIG_LOCATIONS)

    for path in candidate_paths:
        if path.exists() and path.is_file():
            try:
                payload = _load_json(path)
            except (OSError, ValueError) as exc:
                raise ValueError(f"Unable to read configuration from {path}: {exc}") from exc
            mapped: Dict[str, Any] = {}
            for key in ("shorturl_base", "paste_base", "token"):
                if key in payload:
                    mapped[key] = payload[key]
            settings = settings.with_overrides(**mapped)
            break

    # Environment overrides
    env_overrides = {
        "shorturl_base": os.environ.get("ICAKAD_SHORTURL_BASE"),
        "paste_base": os.environ.get("ICAKAD_PASTE_BASE"),
        "token": os.environ.get("ICAKAD_TOKEN"),
    }
    settings = settings.with_overrides(**env_overrides)

    # Direct call overrides win last.
    settings = settings.with_overrides(
        shorturl_base=shorturl_base,
        paste_base=paste_base,
        token=token,
    )

    return settings



"""Публичен интерфейс за инструментите на пакета icakad."""

from __future__ import annotations

from pathlib import Path
from typing import Any, Dict, Optional, Union

from .ai import AI
from .common import print_json, resolve_text_input, write_json, write_text
from .config import Settings, load_settings
from .paste import PasteClient
from .shorturl import ShortURLClient

__all__ = [
    "AI",
    "Settings",
    "load_settings",
    "add_short_link",
    "list_short_links",
    "update_short_link",
    "delete_short_link",
    "list_pastes",
    "create_paste",
    "fetch_paste",
    "print_json",
]

__version__ = "0.1.4"


# ----------------------------------------------------------------- factories
def _client_from_settings(
    *,
    settings: Optional[Settings] = None,
    token: Optional[str] = None,
    shorturl_base: Optional[str] = None,
    timeout: Optional[int] = None,
) -> ShortURLClient:
    cfg = settings or load_settings(token=token, shorturl_base=shorturl_base)
    resolved_timeout = ShortURLClient.timeout if timeout is None else timeout
    return ShortURLClient(
        base_url=cfg.shorturl_base,
        token=cfg.token,
        timeout=resolved_timeout,
    )


def _paste_client_from_settings(
    *,
    settings: Optional[Settings] = None,
    paste_base: Optional[str] = None,
    token: Optional[str] = None,
    timeout: Optional[int] = None,
) -> PasteClient:
    cfg = settings or load_settings(paste_base=paste_base, token=token)
    resolved_timeout = PasteClient.timeout if timeout is None else timeout
    return PasteClient(
        base_url=cfg.paste_base,
        token=cfg.token,
        timeout=resolved_timeout,
    )


# -------------------------------------------------------------- short links
def add_short_link(
    slug: str,
    url: str,
    *,
    save_to: Optional[Union[str, Path]] = None,
    settings: Optional[Settings] = None,
    **overrides: Any,
) -> Dict[str, Any]:
    client = _client_from_settings(settings=settings, **overrides)
    result = client.add_link(slug, url)
    if save_to:
        write_json(result, save_to)
    return result


def list_short_links(
    *,
    save_to: Optional[Union[str, Path]] = None,
    settings: Optional[Settings] = None,
    print_output: bool = True,
    **overrides: Any,
) -> Dict[str, str]:
    client = _client_from_settings(settings=settings, **overrides)
    links = client.list_links()
    if save_to:
        write_json(links, save_to)
    if print_output:
        print_json(links)
    return links


def update_short_link(
    slug: str,
    url: str,
    *,
    save_to: Optional[Union[str, Path]] = None,
    settings: Optional[Settings] = None,
    **overrides: Any,
) -> Dict[str, Any]:
    client = _client_from_settings(settings=settings, **overrides)
    result = client.edit_link(slug, url)
    if save_to:
        write_json(result, save_to)
    return result


def delete_short_link(
    slug: str,
    *,
    save_to: Optional[Union[str, Path]] = None,
    settings: Optional[Settings] = None,
    **overrides: Any,
) -> Dict[str, Any]:
    client = _client_from_settings(settings=settings, **overrides)
    result = client.delete_link(slug)
    if save_to:
        write_json(result, save_to)
    return result


# ------------------------------------------------------------------- pastes
def create_paste(
    *,
    text: Optional[str] = None,
    text_file: Optional[Union[str, Path]] = None,
    paste_id: Optional[str] = None,
    ttl: Optional[int] = None,
    as_plaintext: bool = False,
    save_to: Optional[Union[str, Path]] = None,
    settings: Optional[Settings] = None,
    **overrides: Any,
) -> Dict[str, Any]:
    client = _paste_client_from_settings(settings=settings, **overrides)
    body = resolve_text_input(text=text, text_file=text_file)
    result = client.create_paste(
        body,
        paste_id=paste_id,
        ttl=ttl,
        as_plaintext=as_plaintext,
    )
    if save_to:
        write_json(result, save_to)
    return result


def list_pastes(
    *,
    save_to: Optional[Union[str, Path]] = None,
    settings: Optional[Settings] = None,
    print_output: bool = True,
    **overrides: Any,
) -> Dict[str, Any]:
    client = _paste_client_from_settings(settings=settings, **overrides)
    result = client.list_pastes()
    if save_to:
        write_json(result, save_to)
    if print_output:
        print_json(result)
    return result


def fetch_paste(
    paste_id: str,
    *,
    raw: bool = False,
    save_to: Optional[Union[str, Path]] = None,
    settings: Optional[Settings] = None,
    **overrides: Any,
) -> Any:
    client = _paste_client_from_settings(settings=settings, **overrides)
    result = client.fetch_paste(paste_id, raw=raw)
    if save_to:
        if raw:
            write_text(result, save_to)
        else:
            write_json(result, save_to)
    return result



"""Минимален клиент към LLM работника на icakad."""

from __future__ import annotations

from typing import Dict, Iterable, List, Mapping, MutableMapping, Optional

import requests


Message = Mapping[str, str]

__all__ = ["AI", "ask"]


class AI:
    """Обвивка за изпращане на съобщения към LLM през HTTP."""

    default_url: str = "https://llama.icakad.workers.dev/"
    default_timeout: float = 20.0
    _default_headers: Dict[str, str] = {"Content-Type": "application/json"}

    @classmethod
    def ask(
        cls,
        prompt: str,
        *,
        messages: Optional[Iterable[Message]] = None,
        url: Optional[str] = None,
        timeout: Optional[float] = None,
        session: Optional[requests.sessions.Session] = None,
    ) -> str:
        """Изпраща *prompt* и връща отговора от работника."""

        payload = {"messages": cls._build_messages(prompt, messages)}
        target_url = (url or cls.default_url).rstrip("/") + "/"
        request_timeout = cls.default_timeout if timeout is None else float(timeout)

        sender = session.post if session is not None else requests.post
        response = sender(
            target_url,
            json=payload,
            headers=dict(cls._default_headers),
            timeout=request_timeout,
        )
        response.raise_for_status()
        data = response.json()
        if not isinstance(data, MutableMapping) or "response" not in data:
            raise ValueError("LLM worker returned an unexpected payload")
        return str(data["response"])

    @staticmethod
    def _build_messages(
        prompt: str,
        messages: Optional[Iterable[Message]],
    ) -> List[Dict[str, str]]:
        if messages is not None:
            normalized: List[Dict[str, str]] = []
            for item in messages:
                if not isinstance(item, Mapping):
                    raise TypeError("messages must contain mapping items")
                role = item.get("role")
                content = item.get("content")
                if role is None or content is None:
                    raise ValueError("Each message requires 'role' and 'content'")
                normalized.append({"role": str(role), "content": str(content)})
            if not normalized:
                raise ValueError("messages cannot be empty")
            return normalized

        if not isinstance(prompt, str) or not prompt.strip():
            raise ValueError("prompt must be a non-empty string")
        return [{"role": "user", "content": prompt}]


def ask(
    prompt: str,
    *,
    messages: Optional[Iterable[Message]] = None,
    url: Optional[str] = None,
    timeout: Optional[float] = None,
    session: Optional[requests.sessions.Session] = None,
) -> str:
    """Улеснена обвивка за :meth:`AI.ask` достъпна на ниво модул."""

    return AI.ask(
        prompt,
        messages=messages,
        url=url,
        timeout=timeout,
        session=session,
    )



"""Command-line interface for the icakad package."""

from __future__ import annotations

import argparse
import sys
from typing import Any, Optional, Sequence

from . import (
    add_short_link,
    create_paste,
    delete_short_link,
    fetch_paste,
    list_pastes,
    list_short_links,
    print_json,
    update_short_link,
)
from .common import resolve_text_input


def build_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog="icakad",
        description="Interact with the icakad shorturl and paste services.",
    )
    parser.add_argument("--config", dest="config_path", help="Path to a config JSON file.")
    parser.add_argument("--token", help="Bearer token used for authenticated endpoints.")
    parser.add_argument("--shorturl-base", help="Override the short URL API base URL.")
    parser.add_argument("--paste-base", help="Override the paste API base URL.")

    subparsers = parser.add_subparsers(dest="command")

    # ---------------------------------------------------------------- shorturl
    shorturl_parser = subparsers.add_parser("shorturl", help="Short URL operations")
    shorturl_sub = shorturl_parser.add_subparsers(dest="action")

    shorturl_add = shorturl_sub.add_parser("add", help="Create or overwrite a slug")
    shorturl_add.add_argument("slug", help="Slug used in the short URL")
    shorturl_add.add_argument("url", help="Destination URL")
    shorturl_add.add_argument("--output", help="Write the API response to this JSON file.")
    shorturl_add.add_argument("--quiet", action="store_true", help="Suppress stdout output.")

    shorturl_edit = shorturl_sub.add_parser("update", help="Update the target URL for a slug")
    shorturl_edit.add_argument("slug", help="Slug to update")
    shorturl_edit.add_argument("url", help="New destination URL")
    shorturl_edit.add_argument("--output", help="Write the API response to this JSON file.")
    shorturl_edit.add_argument("--quiet", action="store_true", help="Suppress stdout output.")

    shorturl_del = shorturl_sub.add_parser("delete", help="Delete a slug from the service")
    shorturl_del.add_argument("slug", help="Slug to remove")
    shorturl_del.add_argument("--output", help="Write the API response to this JSON file.")
    shorturl_del.add_argument("--quiet", action="store_true", help="Suppress stdout output.")

    shorturl_list = shorturl_sub.add_parser("list", help="List all known short URLs")
    shorturl_list.add_argument("--output", help="Write the results to a JSON file.")
    shorturl_list.add_argument("--quiet", action="store_true", help="Suppress stdout output.")

    # ------------------------------------------------------------------- paste
    paste_parser = subparsers.add_parser("paste", help="Pastebin operations")
    paste_sub = paste_parser.add_subparsers(dest="action")

    paste_create = paste_sub.add_parser("create", help="Create a new paste")
    group = paste_create.add_mutually_exclusive_group(required=True)
    group.add_argument("--text", help="Inline text for the paste.")
    group.add_argument("--text-file", help="Read paste text from a file.")
    paste_create.add_argument("--id", dest="paste_id", help="Provide a custom paste identifier.")
    paste_create.add_argument(
        "--ttl",
        type=int,
        help="Seconds before the paste expires. Omit for no expiration.",
    )
    paste_create.add_argument(
        "--plain",
        action="store_true",
        help="Send the payload as text/plain instead of JSON.",
    )
    paste_create.add_argument("--output", help="Write the API response to this JSON file.")
    paste_create.add_argument("--quiet", action="store_true", help="Suppress stdout output.")

    paste_get = paste_sub.add_parser("get", help="Fetch a paste by ID")
    paste_get.add_argument("paste_id", help="Paste identifier to fetch")
    paste_get.add_argument("--raw", action="store_true", help="Return the raw text instead of metadata.")
    paste_get.add_argument("--output", help="Write the result to a file.")
    paste_get.add_argument("--quiet", action="store_true", help="Suppress stdout output.")

    paste_list = paste_sub.add_parser("list", help="List all pastes with metadata")
    paste_list.add_argument("--output", help="Write the results to a JSON file.")
    paste_list.add_argument("--quiet", action="store_true", help="Suppress stdout output.")

    return parser


def _common_kwargs(args: argparse.Namespace) -> dict[str, Any]:
    return {
        "config_path": getattr(args, "config_path", None),
        "token": getattr(args, "token", None),
        "base_url": getattr(args, "shorturl_base", None),
    }


def _paste_kwargs(args: argparse.Namespace) -> dict[str, Any]:
    return {
        "config_path": getattr(args, "config_path", None),
        "token": getattr(args, "token", None),
        "base_url": getattr(args, "paste_base", None),
    }


def _print_result(result: Any, quiet: bool, raw: bool = False) -> None:
    if quiet:
        return
    if raw and isinstance(result, str):
        print(result)
    else:
        print_json(result)


def main(argv: Optional[Sequence[str]] = None) -> int:
    parser = build_parser()
    args = parser.parse_args(argv)

    if not args.command:
        parser.print_help()
        return 0

    if args.command == "shorturl":
        if args.action == "add":
            result = add_short_link(
                args.slug,
                args.url,
                save_to=args.output,
                **_common_kwargs(args),
            )
            _print_result(result, args.quiet)
            return 0
        if args.action == "update":
            result = update_short_link(
                args.slug,
                args.url,
                save_to=args.output,
                **_common_kwargs(args),
            )
            _print_result(result, args.quiet)
            return 0
        if args.action == "delete":
            result = delete_short_link(
                args.slug,
                save_to=args.output,
                **_common_kwargs(args),
            )
            _print_result(result, args.quiet)
            return 0
        if args.action == "list":
            result = list_short_links(
                save_to=args.output,
                print_output=not args.quiet,
                **_common_kwargs(args),
            )
            return 0
        parser.error("Please provide a shorturl action (add, update, delete, list).")

    if args.command == "paste":
        if args.action == "create":
            text = resolve_text_input(text=args.text, text_file=args.text_file)
            result = create_paste(
                text=text,
                paste_id=args.paste_id,
                ttl=args.ttl,
                as_plaintext=args.plain,
                save_to=args.output,
                **_paste_kwargs(args),
            )
            _print_result(result, args.quiet)
            return 0
        if args.action == "get":
            result = fetch_paste(
                args.paste_id,
                raw=args.raw,
                save_to=args.output,
                **_paste_kwargs(args),
            )
            _print_result(result, args.quiet, raw=args.raw)
            return 0
        if args.action == "list":
            result = list_pastes(
                save_to=args.output,
                print_output=not args.quiet,
                **_paste_kwargs(args),
            )
            return 0
        parser.error("Please provide a paste action (create, get, list).")

    parser.error("Unknown command. Use --help for usage details.")
    return 1


if __name__ == "__main__":  # pragma: no cover
    sys.exit(main())



"""Thin client for the icakad paste worker JSON API."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any, Dict, Optional, Union

import requests
from requests import Response, Session

DEFAULT_TIMEOUT = 10


class PasteError(RuntimeError):
    """Raised when the paste API reports a failure."""


@dataclass
class PasteClient:
    """Small helper around the paste worker endpoints."""

    base_url: str
    token: Optional[str] = None
    timeout: int = DEFAULT_TIMEOUT
    session: Optional[Session] = None
    _session: Session = field(init=False, repr=False)

    def __post_init__(self) -> None:
        self.base_url = self.base_url.rstrip("/")
        self._session = self.session or requests.Session()

    # ------------------------------------------------------------------ utils
    def _headers(self, *, content_type: Optional[str] = None) -> Dict[str, str]:
        headers: Dict[str, str] = {"Accept": "application/json"}
        if content_type:
            headers["Content-Type"] = content_type
        if self.token:
            headers["Authorization"] = f"Bearer {self.token}"
        return headers

    def _error_message(self, response: Response) -> str:
        body = (response.text or "").strip()
        if body:
            return f"{response.status_code}: {body}"
        return f"{response.status_code}: {response.reason}"

    def _json(self, response: Response) -> Dict[str, Any]:
        try:
            response.raise_for_status()
        except requests.HTTPError as exc:
            raise PasteError(self._error_message(response)) from exc
        try:
            return response.json()
        except ValueError as exc:
            raise PasteError("Paste API returned invalid JSON") from exc

    # ---------------------------------------------------------------- API ops
    def create_paste(
        self,
        text: str,
        *,
        paste_id: Optional[str] = None,
        ttl: Optional[int] = None,
        as_plaintext: bool = False,
    ) -> Dict[str, Any]:
        if not isinstance(text, str) or not text:
            raise ValueError("text must be a non-empty string")

        params: Dict[str, Any] = {}
        if paste_id:
            params["id"] = paste_id
        if ttl is not None:
            params["ttl"] = int(ttl)

        url = f"{self.base_url}/api/paste"

        if as_plaintext:
            headers = self._headers(content_type="text/plain; charset=utf-8")
            response = self._session.post(
                url,
                params=params,
                data=text,
                headers=headers,
                timeout=self.timeout,
            )
        else:
            headers = self._headers(content_type="application/json")
            response = self._session.post(
                url,
                params=params,
                json={"text": text},
                headers=headers,
                timeout=self.timeout,
            )
        return self._json(response)

    def fetch_paste(self, paste_id: str, *, raw: bool = False) -> Union[str, Dict[str, Any]]:
        url = f"{self.base_url}/raw/{paste_id}"
        response = self._session.get(
            url,
            headers=self._headers(),
            timeout=self.timeout,
        )
        try:
            response.raise_for_status()
        except requests.HTTPError as exc:
            raise PasteError(self._error_message(response)) from exc

        text = response.text
        if raw:
            return text

        details: Dict[str, Any] = {
            "id": paste_id,
            "url": f"{self.base_url}/{paste_id}",
            "text": text,
        }

        try:
            listing = self.list_pastes()
        except PasteError:
            return details

        pastes = listing.get("pastes") if isinstance(listing, dict) else None
        if isinstance(pastes, list):
            for item in pastes:
                if isinstance(item, dict) and item.get("id") == paste_id:
                    details.update({k: v for k, v in item.items() if k != "text"})
                    break
        return details

    def list_pastes(self) -> Dict[str, Any]:
        url = f"{self.base_url}/api/list"
        response = self._session.get(
            url,
            headers=self._headers(),
            timeout=self.timeout,
        )
        return self._json(response)



"""Съвместимост със старото име `shortlink`.

Този модул е оставен за потребители на по-стария API. Той просто
препраща всички публични символи към :mod:`icakad.shorturl` и вдига
``DeprecationWarning`` при импорт.
"""

from __future__ import annotations

import warnings

from . import shorturl as _shorturl

globals().update(
    {
        name: getattr(_shorturl, name)
        for name in dir(_shorturl)
        if not name.startswith("_")
    }
)

warnings.warn(
    "`icakad.shortlink` е отложено в полза на `icakad.shorturl`. "
    "Моля, обновете импортите си.",
    DeprecationWarning,
    stacklevel=2,
)

__all__ = [name for name in dir(_shorturl) if not name.startswith("_")]



"""Utility helpers shared across icakad modules."""

from __future__ import annotations

import json
from pathlib import Path
from typing import Any, Iterable, Optional


def ensure_parent(path: Path) -> None:
    """Create parent directories for *path* if they do not exist."""
    path.parent.mkdir(parents=True, exist_ok=True)


def write_json(data: Any, destination: str | Path) -> Path:
    """Serialize *data* to JSON and write it to *destination*."""
    target = Path(destination).expanduser().resolve()
    ensure_parent(target)
    with target.open("w", encoding="utf-8") as fh:
        json.dump(data, fh, indent=2, ensure_ascii=False)
    return target


def write_text(text: str, destination: str | Path) -> Path:
    """Write plain text content to *destination*."""
    target = Path(destination).expanduser().resolve()
    ensure_parent(target)
    with target.open("w", encoding="utf-8") as fh:
        fh.write(text)
    return target


def read_text_file(path: str | Path) -> str:
    """Read UTF-8 content from *path*."""
    target = Path(path).expanduser().resolve()
    with target.open("r", encoding="utf-8") as fh:
        return fh.read()


def resolve_text_input(
    *,
    text: Optional[str] = None,
    text_file: Optional[str | Path] = None,
    allow_empty: bool = False,
) -> str:
    """Return text from either an inline value or a file path."""
    if text is not None:
        candidate = text.strip("\n")
    elif text_file is not None:
        candidate = read_text_file(text_file)
    else:
        raise ValueError("Provide text=... or text_file=... to supply content.")

    if not candidate and not allow_empty:
        raise ValueError("The supplied text is empty.")
    return candidate


def print_json(data: Any) -> None:
    """Pretty-print a JSON-compatible structure."""
    print(json.dumps(data, indent=2, ensure_ascii=False))


def comma_separated(values: Iterable[str]) -> str:
    """Join a series of strings into a comma-separated sentence."""
    return ", ".join(str(v) for v in values)



"""Клиент за Cloudflare worker-а, който управлява късите линкове."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Dict, List, Optional

import requests
from requests import Response, Session

DEFAULT_TIMEOUT = 10


class ShortURLError(RuntimeError):
    """Фатална грешка, върната от shorturl API."""


def _extract_items(payload: object) -> List[Dict[str, object]]:
    if isinstance(payload, dict):
        items = payload.get("items") or payload.get("list")
        if isinstance(items, list):
            return [item for item in items if isinstance(item, dict)]
        return []
    if isinstance(payload, list):
        return [item for item in payload if isinstance(item, dict)]
    return []


@dataclass
class ShortURLClient:
    """Лек HTTP клиент за CRUD операции върху shorturl работника."""

    base_url: str
    token: Optional[str] = None
    timeout: int = DEFAULT_TIMEOUT
    session: Optional[Session] = None
    _session: Session = field(init=False, repr=False)

    def __post_init__(self) -> None:
        self.base_url = self.base_url.rstrip("/")
        self._session = self.session or requests.Session()

    # ---------------------------------------------------------------- utils
    def _headers(self) -> Dict[str, str]:
        headers = {"Content-Type": "application/json"}
        if self.token:
            headers["Authorization"] = f"Bearer {self.token}"
        return headers

    def _request(self, method: str, path: str, **kwargs: object) -> Response:
        url = f"{self.base_url}{path}"
        session_method = getattr(self._session, method)
        response = session_method(
            url,
            headers=self._headers(),
            timeout=self.timeout,
            **kwargs,
        )
        try:
            response.raise_for_status()
        except requests.HTTPError as exc:
            raise ShortURLError(f"{response.status_code}: {response.text}") from exc
        return response

    # ----------------------------------------------------------- API methods
    def add_link(self, slug: str, url: str) -> Dict[str, object]:
        payload = {"slug": slug, "url": url}
        response = self._request("post", "/api", json=payload)
        return self._json(response)

    def edit_link(self, slug: str, url: str) -> Dict[str, object]:
        payload = {"slug": slug, "url": url}
        response = self._request("post", f"/api/{slug}", json=payload)
        return self._json(response)

    def delete_link(self, slug: str) -> Dict[str, object]:
        response = self._request("delete", f"/api/{slug}")
        return self._json(response)

    def list_links(self) -> Dict[str, str]:
        response = self._request("get", "/api")
        try:
            payload = response.json()
        except ValueError as exc:
            raise ShortURLError("ShortURL API returned invalid JSON") from exc

        links: Dict[str, str] = {}
        for item in _extract_items(payload):
            slug = (
                item.get("slug")
                or item.get("key")
                or item.get("id")
                or item.get("name")
            )
            url = item.get("url") or item.get("value")
            if isinstance(slug, str) and isinstance(url, str):
                links[slug] = url
        return links

    # --------------------------------------------------------------- helpers
    def _json(self, response: Response) -> Dict[str, object]:
        try:
            data = response.json()
        except ValueError as exc:
            raise ShortURLError("ShortURL API returned invalid JSON") from exc
        if not isinstance(data, dict):
            raise ShortURLError("ShortURL API returned an unexpected payload")
        return data



"""Тестове за симулиран разговор между два AI клиента."""

from __future__ import annotations

import unittest
from contextlib import redirect_stdout
from io import StringIO
from typing import Any, Dict, List

from icakad import ai as ai_module
from icakad.ai import AI


class _DummyResponse:
    def __init__(self, reply: str) -> None:
        self._reply = reply
        self.status_code = 200

    def raise_for_status(self) -> None:
        return None

    def json(self) -> Dict[str, str]:
        return {"response": self._reply}


class _DummySession:
    def __init__(self) -> None:
        self.turn = 0
        self.posts: List[Dict[str, Any]] = []

    def post(self, url: str, *, json: Dict[str, Any], headers, timeout):
        self.posts.append({"url": url, "payload": json, "headers": headers, "timeout": timeout})
        prompt = json["messages"][-1]["content"]
        responder = "AI-2" if self.turn % 2 == 0 else "AI-1"
        listener = "AI-1" if responder == "AI-2" else "AI-2"
        reply = f"{responder} към {listener}: получих '{prompt}'"
        self.turn += 1
        return _DummyResponse(reply)


class AIConversationTests(unittest.TestCase):
    def test_two_ai_clients_exchange_ten_messages(self) -> None:
        session = _DummySession()
        transcript: List[str] = []
        prompt = "AI-1 към AI-2: Здрасти!"
        stdout_buffer = StringIO()

        with redirect_stdout(stdout_buffer):
            for _ in range(10):
                reply = AI.ask(prompt, session=session)
                transcript.append(reply)
                print(reply)
                prompt = reply

        printed_lines = [line for line in stdout_buffer.getvalue().splitlines() if line]
        self.assertEqual(transcript, printed_lines)
        self.assertEqual(len(transcript), 10)

        expected_speakers = ["AI-2", "AI-1"] * 5
        for reply, speaker in zip(transcript, expected_speakers):
            self.assertTrue(reply.startswith(speaker))

        self.assertEqual(len(session.posts), 10)
        for recorded in session.posts:
            self.assertEqual(recorded["url"], "https://llama.icakad.workers.dev/")
            self.assertIn("messages", recorded["payload"])
            self.assertEqual(recorded["headers"], {"Content-Type": "application/json"})
            self.assertEqual(recorded["timeout"], 20.0)

    def test_module_level_ask_is_available(self) -> None:
        session = _DummySession()
        reply = ai_module.ask("Здравей", session=session)

        self.assertIsInstance(reply, str)
        self.assertEqual(len(session.posts), 1)
        recorded = session.posts[0]
        self.assertEqual(recorded["payload"]["messages"], [{"role": "user", "content": "Здравей"}])


if __name__ == "__main__":
    unittest.main()
